% This script is going to be used to implement the Verlet algorithm for
% simulating the motion of a 2 particle system.

function [X,V] = Verlet(N,T,dt)

close all
clc

% The number of particles in the system
%N = 2;

% The diameter of the particle
sigma = .05;

% Boundary of the box
xl = 0;
xr = 1;

% velocity range
v_range = .005;

% The time and time discretization to run the simulation for
%T = 4;
%dt = .01;


%
% Initialize the particle information
%

% Mass of the particles
M = ones(N,2);

% Position of the particles
x = xl + ((xr-xl).*rand(N,2));

% Velocity of the particles
v = v_range.*rand(N,2);

% Time vector
t = (0:dt:T)';

% The total position vectors
X = zeros(N,2,length(t));

% The total velocity vectors
V = zeros(N,2,length(t));

% Index initial points
X(:,:,1) = x;
V(:,:,1) = v;

% Force and Kinetic/Potential energy in the system
F = zeros(N,2);
Up = zeros(size(t));
Ukt = zeros(size(t));
Ut = zeros(size(t));


% Perform the first force calculation
for iF=1:N
    for jF=iF+1:N

        [condition, r] = pbc(X(iF,:,1),X(jF,:,1),sigma);
        
        %  Check to make sure that it is within the cutoff radius
        if(condition)
            % Note this is the force with respect to epsilon
            % Claulcate the i and j forces and apply them to the i and j
            % particles
            Fij = ((24)./(sigma.^2)).*(2.*((sigma./norm(r)).^14) - ((sigma./norm(r)).^8));
            
            % Decompose the forces
            Fx = Fij.*(r(1)./norm(r));
            Fy = Fij.*(r(2)./norm(r));
            
            % Manage the x forces
            F(iF,:) = F(iF,:) + [-Fx, -Fy];
            F(jF,:) = F(jF,:) + [Fx, Fy];
        end
    end
end

%figure(1)
%hold on
% Plot all the particles
%plot(X(:,1,1),X(:,2,2),'*');


% Now for the Verlet algorithm
for iT=2:length(t)
    
    % FIrst compute the first part of the velocity and position vectors
    X(:,:,iT) = X(:,:,iT-1) + (V(:,:,iT-1).*dt) + ((F.*(dt.^2))./(2*M));
    V(:,:,iT) = V(:,:,iT-1) + (F.*dt)./(2*M);
    
    % Zero out the forces
    F = zeros(N,2);
    
    % Calculate the force of the next time step
    for iF=1:N
        for jF=iF+1:N

            [condition, r] = pbc(X(iF,:,iT-1),X(jF,:,iT-1),sigma);

            %  Check to make sure that it is within the cutoff radius
            if(condition)
                 % Note this is the force with respect to epsilon
                % Claulcate the i and j forces and apply them to the i and j
                % particles
                Fij = ((24)./(sigma.^2)).*(2.*((sigma./norm(r)).^14) - ((sigma./norm(r)).^8));

                % Decompose the forces
                Fx = Fij.*(r(1)./norm(r));
                Fy = Fij.*(r(2)./norm(r));

                % Manage the x forces
                F(iF,:) = F(iF,:) + [-Fx, -Fy];
                F(jF,:) = F(jF,:) + [Fx, Fy];
            end
        end
    end
 
    % Compute the new velocity
    V(:,:,iT) = V(:,:,iT) + (F.*dt)./(2*M);
    
    % If the new position is outside of our box we need to adjust it for 
    % periodic boundary conditions
    for iF=1:N
        if((X(iF,1,iT) < xl) || (xr < X(iF,1,iT)))
            X(iF,1,iT) = mod(X(iF,1,iT),xr-xl);
            V(iF,1,iT) = V(iF,1,iT);
        end
        
        if((X(iF,2,iT) < xl) || (xr < X(iF,2,iT)))
            X(iF,2,iT) = mod(X(iF,2,iT),xr-xl);
            V(iF,2,iT) = V(iF,2,iT);
        end
    end
    
    % At this point we can calculate the energies
    % Calculate the force of the next time step
    for iF=1:N
        for jF=iF+1:N

            % Determine the distance between the particles
            [condition, r] = pbc(X(iF,:,iT),X(jF,:,iT),sigma);
        
            %  Check to make sure that it is within the cutoff radius
            if(condition)
                % Note this is the Potential with respect to epsilon
                % Claulcate the i and j forces and apply them to the i and j
                % particles
                Up(iT) = Up(iT) + (4).*(((sigma./norm(r)).^12) - ((sigma./norm(r)).^6));
            end
        end
        % And Calculate the kinetic energy
        Ukt(iT) = Ukt(iT) + .5*norm(V(iF,:,iT)).^2;
    end
    
    %plot(X(:,1,iT),X(:,2,iT),'*');

end

plot_colors = {'','r','g','b','c','m','y','k'};

figure(1)
hold on
for iF=1:N
    plot(reshape(X(iF,1,:),length(t),1),reshape(X(iF,2,:),length(t),1),plot_colors{1,mod(iF,7)+1},'LineWidth',1.5);
end
xlabel('x','FontSize',16);
ylabel('y','FontSize',16);
title('2D particle movement in a box','FontSize',16);
set(gca,'FontSize',16)
hold off



% Plot at the end
% figure(1)
% hold on
% plot(r1t(:,1),r1t(:,2),'ro-');
% plot(r2t(:,1),r2t(:,2),'bo');
% hold off
% 
% 
% figure(2)
% hold on
% xlabel('time');
% ylabel('Energy');
% title('Energy vs. Time');
% plot(t,Up,'r','LineWidth',1.5);
% plot(t,Uk1,'b','LineWidth',1.5);
% plot(t,Uk2,'c','LineWidth',1.5);
% plot(t,Ukt,'g','LineWidth',1.5);
% plot(t,Ut,'k','LineWidth',1.5);
% legend('Potential','Kinetic particle 1','Kinetic particle 2','total Kinetic','total');
% hold off
    
    



